/***
 *
 * Note that this localization is alredy included by default.
 * Simply call Date.setLocale(<code>)
 *
 * var locale = Date.getLocale(<code>) will return this object, which
 * can be tweaked to change the behavior of parsing/formatting in the locales.
 *
 * locale.addFormat adds a date format (see this file for examples).
 * Special tokens in the date format will be parsed out into regex tokens:
 *
 * {0} is a reference to an entry in locale.optionals. Output: (?:the)?
 * {unit} is a reference to all units. Output: (day|week|month|...)
 * {unit3} is a reference to a specific unit. Output: (hour)
 * {unit3-5} is a reference to a subset of the units array. Output: (hour|day|week)
 * {unit?} "?" makes that token optional. Output: (day|week|month)?
 *
 * {day} Any reference to tokens in them modifiers array will include all with the same name. Output: (yesterday|today|tomorrow)
 *
 * All spaces are optional and will be converted to "\s*"
 *
 * Locale arrays months, weekdays, units, numbers, as well as the "src" field for
 * all entries in the modifiers array follow a special format indicated by a colon:
 *
 * minute:|s  = minute|minutes
 * thicke:n|r = thicken|thicker
 *
 * Additionally in the months, weekdays, units, and numbers array these will be added at indexes that are multiples
 * of the relevant number for retrieval. For example having "sunday:|s" in the units array will result in:
 *
 * units: ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday', 'sundays']
 *
 * When matched, the index will be found using:
 *
 * units.indexOf(match) % 7;
 *
 * Resulting in the correct index with any number of alternates for that entry.
 *
 */

Date.setLocale('it', {
  hasPlural: true,
  months: ['Gennaio','Febbraio','Marzo','Aprile','Maggio','Giugno','Luglio','Agosto','Settembre','Ottobre','Novembre','Dicembre'],
  weekdays: ['Domenica','Luned:ì|i','Marted:ì|i','Mercoled:ì|i','Gioved:ì|i','Venerd:ì|i','Sabato'],
  units: ['millisecond:o|i','second:o|i','minut:o|i','or:a|e','giorn:o|i','settiman:a|e','mes:e|i','ann:o|i'],
  numbers: ['un:|a|o','due','tre','quattro','cinque','sei','sette','otto','nove','dieci'],
  optionals: ["l'|la|il"],
  outputFormat: '{d} {month} {yyyy}',
  pastFormat: '{num} {unit} {sign}',
  futureFormat: '{num} {unit} {sign}',
  modifiers: [
    { name: 'day', src: 'ieri', value: -1 },
    { name: 'day', src: 'oggi', value: 0 },
    { name: 'day', src: 'domani', value: 1 },
    { name: 'day', src: 'dopodomani', value: 2 },
    { name: 'sign', src: 'fa', value: -1 },
    { name: 'sign', src: 'da adesso', value: 1 },
    { name: 'shift', src: 'scors:o|a', value: -1 },
    { name: 'shift', src: 'prossim:o|a', value: 1 }
  ],
  formats: [
    '{num} {unit} {sign}',
    '{weekday?} {date?} {month} {year?}',
    '{0} {unit=5-7} {shift}',
    '{0} {shift} {unit=5-7}'
  ]
});

